use 5.008003;
use strict;
use warnings;
use lib 'lib';
use Test2::Bundle::Numerical qw(:all);

plan(3);
is_finite(1.0, 'is_finite accepts a normal number');
ok(!is_finite(1e999), 'is_finite rejects infinity');
is_finite(0.0, 'is_finite accepts zero');
